

set -ex



rio --help
rio info $RECIPE_DIR/test_data/test.tif
pytest -s -rxs -v -m "not wheel" -k "not (test_warped_vrt_msk_add_alpha or test_warped_vrt_msk_nodata or test_open_https_vsicurl or test_requester_pays or test_ensure_env_decorator_sets_gdal_data_prefix)" tests
exit 0
